/**
 * @fileoverview The visitor keys for the new and updated node types
 * @author Michał Sajnóg <https://github.com/michalsnik>
 * MIT License
 */

"use strict";

const Evk = require("eslint-visitor-keys");

module.exports = Evk.unionWith({
    // Additional Properties.
    ArrayPattern: ["elements", "typeAnnotation"],
    ArrowFunctionExpression: ["typeParameters", "params", "returnType", "body"],
    ClassDeclaration: ["decorators", "id", "typeParameters", "superClass", "superTypeParameters", "implements", "body"],
    ClassExpression: ["decorators", "id", "typeParameters", "superClass", "superTypeParameters", "implements", "body"],
    FunctionDeclaration: ["id", "typeParameters", "params", "returnType", "body"],
    FunctionExpression: ["id", "typeParameters", "params", "returnType", "body"],
    Identifier: ["decorators", "typeAnnotation"],
    MethodDefinition: ["decorators", "key", "value"],
    ObjectPattern: ["properties", "typeAnnotation"],
    NewExpression: ["callee", "typeParameters", "arguments"],
    CallExpression: ["callee", "typeParameters", "arguments"],

    // Additional Nodes.
    ClassProperty: ["decorators", "key", "typeAnnotation", "value"],
    Decorator: ["expression"],
    TSAbstractClassProperty: ["decorators", "key", "typeAnnotation", "value"],
    TSAbstractClassDeclaration: ["decorators", "id", "typeParameters", "superClass", "superTypeParameters", "implements", "body"],
    TSAbstractKeyword: [],
    TSAbstractMethodDefinition: ["key", "value"],
    TSAnyKeyword: [],
    TSArrayType: ["elementType"],
    TSAsyncKeyword: [],
    TSBooleanKeyword: [],
    TSCallSignature: ["typeParameters", "parameters", "typeAnnotation"],
    TSConstructSignature: ["typeParameters", "params", "typeAnnotation"],
    TSConstructorType: ["typeAnnotation", "parameters"],
    TSDeclareKeyword: [],
    TSEmptyBodyDeclareFunction: ["id", "typeParameters", "params", "returnType"],
    TSEmptyBodyFunctionDeclaration: ["id", "typeParameters", "params", "returnType"],
    TSEmptyBodyFunctionExpression: ["id", "typeParameters", "params", "returnType"],
    TSEnumDeclaration: ["members"],
    TSEnumMember: ["id", "initializer"],
    TSExportAssignment: ["expression"],
    TSExportKeyword: [],
    TSImportType: ["parameter", "qualifier", "typeParameters"],
    TSLiteralType: ["literal"],
    TSIndexSignature: ["typeAnnotation", "index"],
    TSInterfaceBody: ["body"],
    TSInterfaceDeclaration: ["id", "typeParameters", "heritage", "body"],
    TSInterfaceHeritage: ["id", "typeParameters"],
    TSFunctionType: ["parameters", "typeAnnotation"],
    TSMethodSignature: ["typeAnnotation", "typeParameters", "key", "params"],
    TSModuleBlock: ["body"],
    TSModuleDeclaration: ["id", "body"],
    TSNamespaceFunctionDeclaration: [],
    TSNonNullExpression: ["expression"],
    TSNeverKeyword: [],
    TSNullKeyword: [],
    TSNumberKeyword: [],
    TSObjectKeyword: [],
    TSParameterProperty: ["parameter"],
    TSPrivateKeyword: [],
    TSPropertySignature: ["typeAnnotation", "key", "initializer"],
    TSProtectedKeyword: [],
    TSPublicKeyword: [],
    TSQualifiedName: ["left", "right"],
    TSQuestionToken: [],
    TSReadonlyKeyword: [],
    TSStaticKeyword: [],
    TSStringKeyword: [],
    TSSymbolKeyword: [],
    TSTypeAnnotation: ["typeAnnotation"],
    TSTypeLiteral: ["members"],
    TSTypeOperator: ["typeAnnotation"],
    TSTypeParameter: ["constraint", "default"],
    TSTypeParameterDeclaration: ["params"],
    TSTypeParameterInstantiation: ["params"],
    TSTypePredicate: ["typeAnnotation", "parameterName"],
    TSTypeReference: ["typeName", "typeParameters"],
    TSTypeQuery: ["exprName"],
    TSUnionType: ["types"],
    TSUndefinedKeyword: [],
    TSVoidKeyword: []
});
